;; 2nd order IIR filter, direct form, R-L-C model
;; Syntax: 	iir2	name,type,fc,q,abs|rel
;; type = lp,bp,hp or bs; fc= cutoff freq
;; q = quality factor; abs: fc in Hertz ; rel: fc in (fs/2)
;; Input: name_in     Output: name




iir2	macro	name,type,fc,q,unit

;;
;;			b0 + b1.z^-1 + b2.z^-2
;;		h(z) = ------------------------
;;			 1 - a1.z^-1 - a2.z^-2
;;
;;
;; DIRECT programmation:
;;
;;
;; y(k) = b0.x(k) + b1.x(k-1) + b2.x(k-2) + a1.y(k-1) + a2.y(k-2)
;;
;; Using dynamic scaling, coefficients are divided by 2:
;;
;; y(k) = 2.[b0/2.x(k) + b1/2.x(k-1) + b2/2.x(k-2) + a1/2.y(k-1) + a2/2.y(k-2)]
;;
;;
;;		xk---o---b0/2-->(+)-->[2]---o---yk
;;		     |		 ^	    |
;;		   [1/z]	 |	  [1/z]
;;		     |	         |	    |
;;		     o---b1/2-->(+)<--a1/2--o
;;		     |	         ^	    |
;;		   [1/z]	 |	  [1/z]
;;		     |		 |	    |
;;		     o---b2/2-->(+)<--a2/2--o
;;
;;
;;
;;
;; Calcul des coefficients en fonction des parametres du filtre 
;; La compilation de "iir_coef" fournit a partir des parametres 
;; "type", " "fe", "f0", et "q" les 5 coefficients kn0, kn1, kn2,
;; kd1, kd2 valant respectivement  b0/2, b1/2, b2/2, a1/2, a2/2
;;
;;
;;

;	var	name,0

	section	vars global
	org	x:
name\_states	dc	0,0,0
	endsec


	if	("unit"=='Hz')||("unit"=='abs')
name\_teta	set	2.*pi*fc/actual_fs
	else
name\_teta	set	2.*pi*fc	
	endif

	
name\_d	set	2.*@tan(name\_teta/(2.*q))/@sin(name\_teta)

name\_beta	set	.5*(1.-(.5*name\_d*@sin(name\_teta)))/(1.+(.5*name\_d*@sin(name\_teta)))

name\_gamma	set	(.5+name\_beta)*@cos(name\_teta)

	if	"type"=='lp'
; Passe-bas:
name\_alpha	set	(.5+name\_beta-name\_gamma)/4
name\_mu	set	2.
name\_sigma	set	1.
	endif

	if	"type"=='hp'
; Passe-haut:
name\_alpha	set	(.5+name\_beta+name\_gamma)/4
name\_mu	set	-2.
name\_sigma	set	1
	endif


	if	"type"=='bp'
;  passe-bande:
name\_alpha	set	(.5-name\_beta)/2.
name\_mu	set	0.
name\_sigma	set	-1.
	endif


	if	"type"=='bs'
; Rejecteur:
name\_alpha	set	(.5+name\_beta)/2
name\_mu	set	-2.*@cos(name\_teta)
name\_sigma	set	1
	endif

;; H(z) exprime par:
;; ( b0 + b1.z^-1 + b2.z^-2 ) / ( 0,5 - a1.z^-1 - a2.z^-2 )
;;
	section	far global
	org	y:
name\_coefs
	dc	@frc(name\_alpha)			; b0/2
	dc	@frc(name\_alpha*name\_mu)		; b1/2
	dc	@frc(name\_alpha*name\_sigma)	; b2/2
	dc	@frc(name\_gamma)			; a1/2
	dc	@frc(-name\_beta)			; a2/2
	endsec

	org	p:
name\_exe	
	cc_rst
	move	#name\_states,r0		; champ x
	move	#-1,m0
	move	#name\_coefs,r4			; champ y
	move	m0,m4
	if	rgin
	move	a,x0
	else
	ldx0	name\_in
	endif
	ldy1	name
	jsr	iir2d
	cc_add	11	
	sta	name
	cc_ac	1
	
	dbgw	IIR2
	dbg	name\_in
	dbg	name


	if		!@def(iir2d)
	section		routines
	global		iir2d
		org	p:

iir2d						   		; x0 = input   y1 = prev output
 	move			y:(r4)+,y0		  	; y0= b0	
 	mpy	x0,y0,a		x:(r0),x1	y:(r4)+,y0 	; a=b0.xk  x1 = xk-1 y0=b1
 	mac	y0,x1,a		x0,x:(r0)+		   	; a+=b1.xk-1 , new xk-1
 	move			x:(r0),x0	y:(r4)+,y0	; x0=xk-2  y0=b2  
 	mac	x0,y0,a		x1,x:(r0)+	y:(r4)+,y0 	; a+= b2.xk-2 , y0=a1, new xk-2
 	mac	y0,y1,a		x:(r0),x0	y:(r4)+,y0 	; a+= a1.yk-1  x0=yk-2 y0=a2
 	mac	x0,y0,a		y1,x:(r0)		  	; a+= a2.yk-2  new yk-2
 	asl	a
	rts
	endsec
	endif



; Used registers for sample execution
	u	r0
	u	m0
	u	r4
	u	m4
	u	x
	u	y
	u	a

	endm
;; DSP56300 ASM Code
;; Written by Jean-Marie ORY
;; Free software restricted to academic use


